import json
import sys

import requests

KOT_API_BASE_URL = 'https://api.kingtime.jp/v1.0'


class SokotRequester():
    def __init__(self):
        self.base_url = KOT_API_BASE_URL
        self.headers = {
            'Content-Type': 'application/json; charset=utf-8',
            'Authorization': 'Bearer {token}'
        }

    def get(self, uri, token):
        self.headers['Authorization'] = self.headers['Authorization'].format(token=token)
        url = self.base_url + uri
        resp = requests.get(url, headers=self.headers)
        resp_json = json.loads(resp.text)
        if 'errors' in resp_json:
            print(resp_json['errors'][0]['message'])
            sys.exit(1)
        return resp_json

    def post(self, uri, token, payload):
        self.headers['Authorization'] = self.headers['Authorization'].format(token=token)
        url = self.base_url + uri
        resp = requests.post(url, headers=self.headers, data=payload)
        resp_json = json.loads(resp.text)
        if 'errors' in resp_json:
            print(resp_json['errors'][0]['message'])
            sys.exit(1)
        return resp_json



from sokot.configuration import SokotConfiguration


class SokotGroup():
    def __init__(self):
        self._config = SokotConfiguration()

    def add(self, name, *members):
        self._config.check()
        self._config.add_group(name, *members)
        return "Group '{}' is added: [{}]".format(name, ', '.join(members))

    def delete(self, name):
        self._config.check()
        self._config.delete_group(name)
        return "Group '{}' is deleted".format(name)

    def ls(self):
        return self._config.list_group()



import fire

from sokot.aggregator import SokotAggretator
from sokot.configuration import SokotConfiguration
from sokot.group import SokotGroup
from sokot.requester import SokotRequester


class Sokot():
    def __init__(self):
        self._config = SokotConfiguration()
        self._requester = SokotRequester()
        self._aggregator = SokotAggretator()
        self.group = SokotGroup()

    def configure(self):
        self._config.create()
        return 'OK'

    def available(self):
        token = self._config.check()
        resp = self._requester.get('/tokens/{}/available'.format(token), token)
        return resp['available']

    def overtime(self, agg_type='sum'):
        """
        sum: チームの残業時間合計
        per_member: sum をメンバー数で割った時間
        """
        self._config.check()
        result = self._aggregator.aggregate(agg_type)
        print(result)


def main():
    fire.Fire(Sokot(), name='sokot')


if __name__ == '__main__':
    main()



import datetime
import getpass
import json
import os
import sys

CONFIGURATION_MESSAGE = """
To start using sokot, run:

sokot configure
""".strip()
SOKOT_CONFIG_DIR = os.path.join(os.environ['HOME'], '.sokot')
TOKEN_PATH = os.path.join(os.environ['HOME'], '.sokot', 'token')
CONFIG_PATH = os.path.join(os.environ['HOME'], '.sokot', 'config.json')


class SokotConfiguration():
    def __init__(self):
        if not os.path.exists(SOKOT_CONFIG_DIR):
            os.mkdir(SOKOT_CONFIG_DIR)

    def _is_configured(self):
        return os.path.exists(TOKEN_PATH) and os.path.exists(CONFIG_PATH)

    def _set_token(self, token):
        f = open(TOKEN_PATH, 'w')
        f.write(token)
        f.close()

    def get_token(self):
        return self._get_token()

    def _get_token(self):
        f = open(TOKEN_PATH, 'r')
        token = f.read()
        f.close()
        return token

    def _set_config(self, config_dict):
        f = open(CONFIG_PATH, 'w')
        f.write(json.dumps(config_dict))
        f.close()

    def _get_config(self):
        if not os.path.exists(CONFIG_PATH):
            return {}
        f = open(CONFIG_PATH, 'r')
        json_str = f.read()
        f.close()
        return json.loads(json_str)

    def check(self):
        if self._is_configured():
            return self._get_token()
        print(CONFIGURATION_MESSAGE)
        sys.exit(1)

    def create(self):
        token = getpass.getpass(prompt='Token:')
        self._set_token(token)
        print("Saved to '{}'\n".format(TOKEN_PATH))

        sprint_length = int(input('Sprint Length (Default: 2):') or 2)
        scrum_start = input('The First Scrum Day (Default: 2018.01.01):') or '2018.01.01'
        config_dict = {
            'sprint_length': sprint_length,
            'scrum_start': scrum_start,
            'groups': {}
        }
        self._set_config(config_dict)
        print("Saved to '{}'".format(CONFIG_PATH))

    def get_scrum_start_day(self):
        config = self._get_config()
        date_str = config.get('scrum_start', None)
        if not date_str:
            return datetime.date.today()
        return datetime.datetime.strptime(date_str, '%Y.%m.%d').date()

    def add_group(self, name, *members):
        config = self._get_config()
        member_list = list(members)
        config['groups'][name] = member_list
        self._set_config(config)

    def delete_group(self, name):
        config = self._get_config()
        config['groups'].pop(name, None)
        self._set_config(config)

    def list_group(self):
        config = self._get_config()
        return config['groups']



import datetime

from beautifultable import BeautifulTable

from sokot.configuration import SokotConfiguration
from sokot.requester import SokotRequester

DAILY_WORKING_API = '/daily-workings?start={}&end={}&additionalFields=currentDateEmployee'


class SokotAggretator():
    def __init__(self):
        self._config = SokotConfiguration()
        self._sprints = self._culc_sprint()
        self._requester = SokotRequester()

    def _culc_sprint(self):
        sprints = []
        start_day = self._config.get_scrum_start_day()
        sprint_start = start_day
        while True:
            if sprint_start > datetime.date.today():
                break
            sprint_end = sprint_start + datetime.timedelta(days=13)
            sprint = (sprint_start, sprint_end)
            sprints.append(sprint)
            sprint_start = sprint_end + datetime.timedelta(days=1)
        return sprints

    def _print_warning(self, record):
        last_name = record['currentDateEmployee']['lastName']
        first_name = record['currentDateEmployee']['firstName']
        error_date = record['date']
        print('{} {}さんの{}の入力にエラーがあります'.format(last_name, first_name, error_date))

    def _aggregate_sprint(self, members, sprint_start, sprint_end):
        token = self._config.get_token()
        resp = self._requester.get(DAILY_WORKING_API.format(sprint_start, sprint_end), token)
        sum_min = 0
        for daily_record in resp:
            for record in daily_record['dailyWorkings']:
                if record['currentDateEmployee']['code'] in members:
                    if record['isError']:
                        self._print_warning(record)
                    else:
                        sum_min += record['overtime']
        return round(sum_min / 60, 2)

    def aggregate(self, agg_type):
        table = BeautifulTable()
        group_names = list(self._config.list_group().keys())
        table.column_headers = ["Sprint No."] + group_names

        sprint_no = 0
        for sprint in self._sprints:
            sprint_no += 1
            sprint_start, sprint_end = sprint[0], sprint[1]
            sprint_str = "Sprint #{} ({} - {})".format(sprint_no, sprint_start, sprint_end)
            group_result = []
            for _, members in self._config.list_group().items():
                result = self._aggregate_sprint(members, sprint_start, sprint_end)
                group_result.append(result)
            table.append_row([sprint_str] + group_result)
        return table



from setuptools import setup

SHORT_DESCRIPTION = """
A tool to get your member's overtime from King of Time for every N weeks.""".strip()

DEPENDENCIES = [
    'fire',
]

TEST_DEPENDENCIES = [
]

VERSION = '0.1.0'
URL = 'https://github.com/showwin/scrum-overtime-kot'

setup(
    name='sokot',
    version=VERSION,
    description=SHORT_DESCRIPTION,
    url=URL,

    author='showwin',
    author_email='showwin_kmc@yahoo.co.jp',
    license='Apache Software License',

    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    entry_points={
        'console_scripts': 'sokot = sokot.main:main'
    },

    keywords='overtime king-of-time scrum python tool cli',

    packages=['sokot'],

    install_requires=DEPENDENCIES,
    tests_require=TEST_DEPENDENCIES,
)
